{% extends "base.html" %}

{% block content %}
<div id="header">
    <!-- Header and Scan Submission Inline -->
  <div class="d-flex justify-content-start align-items-center">
      <h1 class="title" onclick="location.href = '/'">
          <span>LAN</span>scape 
      </h1>
  </div>
</div>
<div class="scroll-container" id="#content">
    <div class="container-fluid my-4">
        <h3>Scan Errors:</h3>
        
        {% for error in data['errors'] %}
            <details>
                <summary>{{ error.get('basic') }}</summary>
                <div class="error-container">
                    <pre>{{ error.get('traceback') }}</pre>
                </div>
            </details>
        {% endfor %}
        <button onclick="location.href='/'" class="btn btn-primary m-2">Home</button>

        <a target="_blank" href="https://github.com/mdennis281/LANscape/issues">
            <button class="btn btn-info m-2">Github issues</button>
        </a>
    </div>   
</div>

{% endblock %}

